function ixsound(action);
%iXsound (IPERSOUND) 
%   Script per eseguire semplici analisi di segnali audio
%   registrati in formato .WAV
%   
%   Il segnale pu essere visualizzato nel dominio del tempo 
%   sia in banda larga (non filtrato) che in bande di ottava,
%   e nel dominio delle frequenze in forma di spettro sonoro
%   (di cui  possibile scegliere la risoluzione spettrale)
%   e di sonogramma.
%
%   Il segnale pu essere poi ascoltato. Quando  attiva la
%   visualizzazione del segnale filtrato viene eseguito il 
%   suono corrispondente
%
%   
%   NB. Per poter eseguire l'analisi spettrale  necessario 
%   il Signal Processing Toolbox, unitamente alla funzione
%   specgram2.
  
%   Based on the original XPSOUND by  
%   Denise L. Chen, 7-27-93
%   Copyright (c) 1984-96 by The MathWorks, Inc.
%   $Revision: 5.7 $  $Date: 1996/10/15 16:19:30 $

global SD_VOLUME SD_NAMES SD_DISP SD_PLOTCMD Z_up Z_dn ZoomFlag Fs Res zd zu SD tz Lp Freq SD_filt
global funcs func_titles

if nargin < 1,
   action = 'initialize';
end;

if strcmp(action,'initialize')
   oldFigNumber=watchon;
   
   %================================================================
   % Set all the global variables
   SD_PLOTCMD = str2mat('plot((zd:zu)/Fs,y(zd:zu));grid on;axis tight;xlabel(''Tempo (s)'');',...
      'plot((zd:zu)/Fs,SD_filt(1,(zd:zu)));grid on;axis tight;xlabel(''Tempo (s)'');',...
      'plot((zd:zu)/Fs,SD_filt(2,(zd:zu)));grid on;axis tight;xlabel(''Tempo (s)'');',...
      'plot((zd:zu)/Fs,SD_filt(3,(zd:zu)));grid on;axis tight;xlabel(''Tempo (s)'');',...
      'plot((zd:zu)/Fs,SD_filt(4,(zd:zu)));grid on;axis tight;xlabel(''Tempo (s)'');',...
      'plot((zd:zu)/Fs,SD_filt(5,(zd:zu)));grid on;axis tight;xlabel(''Tempo (s)'');',...
      'plot((zd:zu)/Fs,SD_filt(6,(zd:zu)));grid on;axis tight;xlabel(''Tempo (s)'');',...
      ['[p,f]=spectrum(y(zd:zu),Res,128,hanning(256),Fs);loglog(f,p(:,1));axis tight; set(gca,''XGrid'',''on'',''YGrid'',''off'');;h=get(gca,''children'');',...
         'delete(h(1:2));xlabel(''Frequenza in Hertz'');'],...
      ['specgram2(y(zd:zu),256,Fs);xlabel(''Tempo (s)'');',...
         'ylabel(''Frequenza (Hz)'');'],...
      ['bar(Lp);grid off; xlabel(''Frequenze centrali di banda (Hz)'');ylabel(''Livello (dB)'');'],...
      ['hist(y,20)']);
   
   SD_DISP = 1;
   Res = 1024;
   ZoomFlag = 'on';
   
   S =  brighten([[zeros(8,2) (3:10)'/10]; prism(56)],1/3);
   if (get(0,'ScreenDepth') == 1)
      S = gray(64);
   end
   
     
   %================================================================
   figNumber = figure('colormap',S,'visible','on','NumberTitle','off', ...
      'Name','Sound analyser','pointer','watch');
   
   wids = 0.26;
   axes('position', [0.1 .22 .9*(.9-wids) .68], ...
      'vis','off');   
   
   %================================================================
   % Information for all buttons
   labelColor=192/255*[1 1 1];
   top=0.92;
   bottom=0.06;
   left=0.75;
   yInitLabelPos=0.90;
   left=0.73;
   labelWid=0.23;
   labelHt=0.05;
   btnWid=0.23;
   btnHt=0.035;
   % Spacing between the label and the button for the same command
   btnOffset=0.003;
   % Spacing between the button and the next command's label
   spacing=0.05;
   
   %=================================================================
   % The CONSOLE frame
   frmBorder=0.02;
   yPos=0.05-frmBorder;
   frmPos=[left-frmBorder yPos btnWid+2*frmBorder 0.9+2*frmBorder];
   h=uicontrol( ...
      'Style','frame', ...
      'Units','normalized', ...
      'Position',frmPos, ...
      'BackgroundColor',[0.5 0.5 0.5]);
   
   %=======================================================================
   
   % The LOAD button
   
   btnNumber = 1;
  yLabelPos=top-(btnNumber-1)*(btnHt+labelHt+spacing);
   labelPos=[left yLabelPos-2*btnHt labelWid 2*btnHt];
   uicontrol( ...
      'Style','text', ...
      'Units','normalized', ...
      'Position',labelPos, ...
      'BackgroundColor',labelColor, ...
      'HorizontalAlignment','left', ...
      'String',' Nome del file .WAV');
   
   % Pop-up menu info
   btnPos=[left yLabelPos-2*labelHt-btnOffset btnWid labelHt];
   
   SD_name=uicontrol('Style','edit',...
         'Units','normalized', ...
         'Position', btnPos, ...
         'Callback','ixsound(''load'')');

   


     
   %=======================================================================
   % The DISPLAY command popup menu
   
   % Menu label info
   btnNumber=2;
   yLabelPos=top-(btnNumber-1)*(btnHt+labelHt+spacing);
   labelPos=[left yLabelPos-labelHt labelWid labelHt];
   uicontrol( ...
      'Style','text', ...
      'Units','normalized', ...
      'Position',labelPos, ...
      'BackgroundColor',labelColor, ...
      'HorizontalAlignment','left', ...
      'String',' Visualizza');
   
   % Pop-up menu info
   btnPos=[left yLabelPos-labelHt-btnHt-btnOffset btnWid btnHt];
   
   % Signal Processing Toolbox is on the path
   if exist('specgram')
      uicontrol('Style','popup',...
         'Units','normalized', ...
         'Position', btnPos, ...
         'String',str2mat('Sequenza temporale','Sequenza a 125 Hz','Sequenza a 250 Hz','Sequenza a 500 Hz','Sequenza a 1 kHz','Sequenza a 2 kHz','Sequenza a 4 kHz','Spettro','Sonogramma','Spettro in ottave','Distribuzione di ampiezza'),...
         'Callback','ixsound(''display'')');
      % Signal Processing Toolbox is not on the path
   else
      uicontrol('Style','text',...
         'Units','normalized', ...
         'Position', btnPos, ...
         'String',str2mat('Sequenza temporale'));
   end;
   
   %=======================================================================
   % The RESOLUTION command popup menu
   
   % Menu label info
   btnNumber=3;
   yLabelPos=top-(btnNumber-1)*(btnHt+labelHt+spacing);
   labelPos=[left yLabelPos-labelHt labelWid labelHt];
   uicontrol( ...
      'Style','text', ...
      'Units','normalized', ...
      'Position',labelPos, ...
      'BackgroundColor',labelColor, ...
      'HorizontalAlignment','left', ...
      'String',' Risoluzione spettro');
   
   % Pop-up menu info
   btnPos=[left yLabelPos-labelHt-btnHt-btnOffset btnWid btnHt];
   
   % Signal Processing Toolbox is on the path
   if exist('spectrum')
      uicontrol('Style','popup',...
         'Units','normalized', ...
         'Position', btnPos, ...
         'String',str2mat('1024','2048','4096','8192','16384'),...
         'Callback','ixsound(''resolution'')');
      % Signal Processing Toolbox is not on the path
   else
      uicontrol('Style','text',...
         'Units','normalized', ...
         'Position', btnPos, ...
         'String',str2mat('Disabled'));
   end;
   
   %=======================================================================
   % The PLAY button
   btnNumber = 4;
   yLabelPos=top-(btnNumber-1)*(btnHt+labelHt+spacing);
   btnPos=[left yLabelPos-labelHt-btnHt-btnOffset btnWid 2*btnHt];
   btnHndl=uicontrol( ...
      'Style','pushbutton', ...
      'Units','normalized', ...
      'Position', btnPos, ...
      'String','Esegui suono', ...
      'Callback','ixsound(''play'')');
   % use try-catch to hide the PLAY button if sound is unavailable
   soundFlag='on';
   eval('sound(0)','soundFlag=''off'';');
   set(btnHndl,'Visible',soundFlag);
   
   %=======================================================================
   % The INFO button
   uicontrol( ...
      'Style','pushbutton', ...
      'Units','normalized', ...
      'Position',[left bottom+2*btnHt+spacing btnWid 2*btnHt], ...
      'String','Info', ...
      'Callback','ixsound(''info'')');
   
   %=======================================================================
   % The CLOSE button
   uicontrol( ...
      'Style','pushbutton', ...
      'Units','normalized', ...
      'Position',[left bottom btnWid 2*btnHt], ...
      'String','Chiudi', ...
      'Callback','ixsound(''close'')');
   
   %=======================================================================
   % The VOLUME slider
   
   SD_VOLUME = uicontrol( ...
      'Units','normal',...
      'Position',[0.125 .04 .8*(.9-wids) .04], ...
      'Style','slider', ...
      'Visible',soundFlag, ...
      'Min',0.01, 'max',1, ...
      'Value',1);
   
   % label volume control
   pos = [0.01 0.04 0.10 0.04];
   uicontrol('style','text', ...
      'Units','normal', ...
      'Position',pos,...
      'String','Volume', ...
      'BackgroundColor',get(gcf,'color'),...
      'Visible',soundFlag, ...
      'ForegroundColor', get(gca,'xcolor'),...
      'Horizontal','left');
   
   %=======================================================================
   % the ZOOM limit control
       
   Z_dn=uicontrol( ...
      'Units','normal',...
      'Position',[0.06 .1 .08 .05], ...
      'Style','edit', ...
      'Visible','off', ...
      'String','0',...
      'Callback','ixsound(''zoomdn'')');
   
   Z_up=uicontrol( ...
      'Units','normal',...
      'Position',[(.88-wids) .1 .08 .05], ...
      'Style','edit', ...
      'Visible','off', ...
      'String','1',...
      'Callback','ixsound(''zoomup'')');

   
   
   %=======================================================================
  
   title('No file');
   drawnow;
   set(gcf,'pointer','arrow');
   
   %=======================================================================
   watchoff(oldFigNumber);
   
   % END of INITIALIZE section
   
   %=======================================================================
   % Load callback.
   
elseif strcmp(action,'load')
   hndl=gco;
   SD_name=get(hndl,'String');

   [y,Fs] = wavread(SD_name);

   y=y/max(abs(y));t=(0:length(y)-1)/Fs;
   SD = y;
   zd = 1;
   zu = length(y);
   % ========================================
   % FILTER SOUND
   wb = waitbar(0,'Sto filtrando il segnale...');
   for i=1:6
   bf=125*(2^(i-1));
   lp=bf/(sqrt(2)); %lowpass frequency
   hp=bf*(sqrt(2)); % highpass frequency
   lo=lp*2/Fs;
   hi=hp*2/Fs;
   FL_ord=2; %filter order
   Wn=[lo hi];
   [B,A] = butter(FL_ord,Wn);
   S = SD';
      for j=1:4
         Sf(i,:)=filter(B,A,S);
         S=Sf(i,:);
      end
   waitbar(i/6)   
end
close(wb)
SD_filt = Sf;
   clear bf lp hp lo hi FL_ord Wn B A i j S Sf
   % =================================================
   % Calculate Octave Band Levels
   p2=SD_filt.^2;
   su=(sum(p2'))';
   Freq = [125 250 500 1000 2000 4000];
   Lp=10*log10(su/((4*10^(-10)*Fs)));
   clear p2 su
   % =================================================
   set(gcf,'UserData',[t(:) y(:)]);
   n = SD_DISP;
   eval([SD_PLOTCMD(n,:)]);
   grid on
   title([int2str(length(y)) ' Samples - ' int2str(Fs) 'Hz']);
   set(Z_dn,'Visible',ZoomFlag)
   set(Z_up,'Visible',ZoomFlag)
   set(Z_up,'String',int2str(zu/Fs))
   
   drawnow;
   
   set(gcf,'pointer','arrow');
   
   %========================================================================
   % Display callback.
   
elseif strcmp(action,'display')
   hndl=gco;
   popStr=get(hndl,'String');
   value=get(hndl,'Value');
   selectStr=deblank(popStr(value,:));
   
          
   SD_DISP = value;
   if SD_DISP == 1
      ZoomFlag = 'on';
   else
      ZoomFlag = 'off';
   end
   set(Z_dn,'Visible',ZoomFlag)
   set(Z_up,'Visible',ZoomFlag)
   y = get(gcf,'UserData');
   if ~isempty(y),t = y(:,1); y = y(:,2);
      set(gcf,'pointer','watch');drawnow;
      titl = get(get(gca,'title'),'String');
      eval([SD_PLOTCMD(value,:)]);
      set(gcf,'pointer','arrow');
      title(titl),drawnow,end;
   
   %========================================================================
   % Resolution callback.
   
elseif strcmp(action,'resolution')
   hndl=gco;
   value=get(hndl,'Value');
   Res = 1024*2^(value-1);
   
            
   y = get(gcf,'UserData');
   if ~isempty(y),t = y(:,1); y = y(:,2);
      set(gcf,'pointer','watch');drawnow;
      titl = get(get(gca,'title'),'String');
      eval([SD_PLOTCMD(SD_DISP,:)]);
      set(gcf,'pointer','arrow');
      title(titl),drawnow,end;

   
   %========================================================================
   % Play callback.
   
elseif strcmp(action,'play')
   
   set(gcf,'pointer','watch');
   
   
   %dat = get(gcf,'UserData');
   %y = dat(:,2);
   if SD_DISP >1 & SD_DISP<8
      sound(SD_filt(SD_DISP-1,(zd:zu))*get(SD_VOLUME,'value'),Fs);
   else   
      sound(SD(zd:zu)*get(SD_VOLUME,'value'),Fs);
   end
   set(gcf,'pointer','arrow');
   
   %========================================================================
   % Zoom callback
elseif strcmp(action,'zoomdn')
   hndl=gco;
   value=str2num(get(hndl,'String'));
   zd = round(Fs*value);
   if zd < 1
      zd = 1;
      set(Z_dn,'String','0')
   end
   if zd > zu
      warndlg('Attenzione! Il limite inferiore  maggiore di quello superiore')
      zd = 1;
      set(Z_dn,'String','0')
   end
   tz = (zd-1:zu-1)/Fs;
   plot(tz,SD(zd:zu));xlabel('Tempo (s)')
   axis tight
   grid on

   
elseif strcmp(action,'zoomup')
   hndl=gco;
   value=str2num(get(hndl,'String'));
   zu = round(Fs*value);
   if zu > length(SD)
      zu = length(SD);
      set(Z_up,'String',int2str(zu/Fs))
   end
   if zu < zd
      warndlg('Attenzione! Il limite superiore  minore di quello inferiore')
      zu = length(SD);
      set(Z_up,'String',int2str(zu/Fs))
   end
   
   tz = (zd-1:zu-1)/Fs;
   plot(tz,SD(zd:zu));xlabel('Tempo (s)')
   axis tight
   grid on
   
   %========================================================================

elseif strcmp(action,'info'),
   helpwin(mfilename)
   
elseif strcmp(action,'close'),
   close(gcf);
   clear global SD_VOLUME SD_NAMES SD_DISP SD_PLOTCMD Fs;
   clear global funcs func_titles;
   
end;    % if strcmp(action, ...
